searchNodes=[{"doc":"Define modules for the efficient generation of cryptographically strong probably unique identifiers (&lt;strong&gt;puid&lt;/strong&gt;s, aka random strings) of specified entropy from various character sets Examples The simplest usage of Puid requires no options. The library adds a generate/0 function for generating puid s: iex&gt; defmodule ( Id , do : use ( Puid ) ) iex&gt; Id . generate ( ) &quot;p3CYi24M8tJNmroTLogO3b&quot; By default, Puid modules generate puid s with at least 128 bits of entropy, making the puid s suitable replacements for uuid s. Character Set The default character set for Puid modules is the Base64 URL and file system safe character set specified in RFC 3548 . Any of the pre-defined character sets from Puid.CharSet can easily be specified using the charset option: iex&gt; defmodule ( HexId , do : use ( Puid , charset : :hex ) ) iex&gt; HexId . generate ( ) &quot;a60dec6d0b71355aa9579bb46c001700&quot; Custom Characters Any sequence of unique, printable characters can be used to generate puid s. iex&gt; defmodule ( DingoSkyId , do : use ( Puid , chars : &quot;dingosky&quot; ) ) iex&gt; DingoSkyId . generate ( ) &quot;yoisknoydoknkoikgoknynkinoknkygdiikoosksyni&quot; iex&gt; defmodule ( UnicodeId , do : use ( Puid , chars : &quot;ŮήιƈŏδεĊħąŕαсτəř&quot; ) ) iex&gt; UnicodeId . generate ( ) &quot;αήήδħƈĊŕąąιŏήąŕħƈδəəήιττδδŕąĊδŕι&quot; Specific Entropy Bits The bits option can be used to specify desired entropy bits. iex&gt; defmodule Password , do : use Puid , bits : 96 , charset : :printable_ascii iex&gt; Password . generate ( ) &quot;0&amp;pu=w+T#~o)N=E&quot; Since the total entropy bits of a puid must be a multiple of the entropy bits per character used, the actual puid bits will be equal to or greater than specified. In the example above, the entropy bits of a Password generated puid is actually 98.32. Total and Risk The amount of entropy can be intuitively specified through the total and risk options. For example, to generate a total of 10 million puid s with a 1 in a quadrillion risk of repeat using :safe32 characters: iex&gt; defmodule ( Safe32Id , do : use ( Puid , total : 1.0e7 , risk : 1.0e15 , charset : :safe32 ) ) iex&gt; Safe32Id . generate ( ) &quot;hjM7md2R9j8D7PNTjBPB&quot; The actual Safe32Id puid entropy bits is 100. Custom Randomness Puid generates puid s using bytes from the function specified with the rand_bytes option. If rand_bytes is not specified, Puid defaults to :crypto.strong_rand_bytes/1 . iex&gt; defmodule ( MyRandBytesId , do : use ( Puid , bits : 96 , charset : :safe32 , rand_bytes : &amp; MyRand . bytes / 1 ) ) iex&gt; MyRandBytesId . generate ( ) &quot;G2jrmPr3mQPBt2gGB3T4&quot; The MyRand.bytes/1 function must be of the form (non_neg_integer) -&gt; binary() Module Functions Puid adds the following 2 functions to each created module: Function Description generate/0 function for generating a puid info/0 Puid.Info struct of module information The Puid.Info struct has the following fields: Field Description chars source character set charset pre-defined Puid.Charset or :custom entropy_bits entropy bits for generated puid s entropy_bits_per_char entropy bits per character for generated puid s ere puid entropy string representation efficiency length puid string length rand_bytes entropy source function iex&gt; defmodule ( AlphanumId , do : use ( Puid , total : 10 e06 , risk : 1.0e15 , charset : :alphanum ) ) iex&gt; AlphanumId . info ( ) % Puid.Info { chars : &quot;ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789&quot; , charset : :alphanum , entropy_bits : 107.18 , entropy_bits_per_char : 5.95 , ere : 0.74 , length : 18 , rand_bytes : &amp; :crypto . strong_rand_bytes / 1 }","ref":"Puid.html","title":"Puid","type":"module"},{"doc":"Pre-defined Puid.CharSet s are specified via an atom charset option during Puid module definition. Example iex&gt; defmodule ( AlphanumId , do : use ( Puid , charset : :alphanum ) ) Pre-defined CharSets :alpha Upper/lower case alphabet ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz :alpha_lower Lower case alphabet abcdefghijklmnopqrstuvwxyz :alpha_upper Upper case alphabet ABCDEFGHIJKLMNOPQRSTUVWXYZ :alphanum Upper/lower case alphabet and numbers ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789 :alphanum_lower Lower case alphabet and numbers abcdefghijklmnopqrstuvwxyz0123456789 :alphanum_upper Upper case alphabet and numbers ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789 :base32 RFC 4648 base32 character set ABCDEFGHIJKLMNOPQRSTUVWXYZ234567 :base32_hex RFC 4648 base32 extended hex character set with lowercase letters 0123456789abcdefghijklmnopqrstuv :base32_hex_upper RFC 4648 base32 extended hex character set 0123456789ABCDEFGHIJKLMNOPQRSTUV :decimal Decimal digits 0123456789 :hex Lowercase hexidecimal 0123456789abcdef :hex_upper Uppercase hexidecimal 0123456789ABCDEF :safe32 Strings that don't look like English words and are easy to parse visually 2346789bdfghjmnpqrtBDFGHJLMNPQRT remove all upper and lower case vowels (including y) remove all numbers that look like letters remove all letters that look like numbers remove all letters that have poor distinction between upper and lower case values :safe64 RFC 4648 file system and URL safe character set ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789-_ :printable_ascii Printable ASCII characters from ?! to ?~ `!&quot;#$%&amp;'()*+,-./0123456789:;&lt;=&gt;?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_`abcdefghijklmnopqrstuvwxyz{|}~`","ref":"Puid.CharSet.html","title":"Puid.CharSet","type":"module"},{"doc":"Return pre-defined Puid.CharSet characters or :undefined Example iex&gt; Puid.CharSet . chars ( :safe32 ) &quot;2346789bdfghjmnpqrtBDFGHJLMNPQRT&quot; iex&gt; Puid.CharSet . chars ( :dne ) :undefined","ref":"Puid.CharSet.html#chars/1","title":"Puid.CharSet.chars/1","type":"function"},{"doc":"Entropy related calculations The implementation is based on mathematical approximations to the solution of what is often referred to as the Birthday Problem .","ref":"Puid.Entropy.html","title":"Puid.Entropy","type":"module"},{"doc":"Entropy bits for generating a total number of instances with the given risk of repeat The total size of the instance pool is 2&lt;sup&gt;bits&lt;/sup&gt;. Example iex&gt; Puid.Entropy . bits ( 10.0e6 , 1.0e12 ) 85.37013046707142","ref":"Puid.Entropy.html#bits/2","title":"Puid.Entropy.bits/2","type":"function"},{"doc":"Entropy bits of a string of len generated from characters charset , where charset is either an pre-defined Puid.CharSet or a string of unique characters. The character set must be comprised of unique symbols, and it is assumed each symbol in the character set has equal probability of occurrence (which maximizes entropy). Example iex&gt; Puid.Entropy . bits_for_len ( 14 , :alphanum ) { :ok , 83 } iex&gt; Puid.Entropy . bits_for_len ( 14 , &quot;dingosky&quot; ) { :ok , 42 }","ref":"Puid.Entropy.html#bits_for_len/2","title":"Puid.Entropy.bits_for_len/2","type":"function"},{"doc":"Same as Puid.Entropy.bits_for_len/2 but either returns the integer bits or raises a Puid.Error Example iex&gt; Puid.Entropy . bits_for_len! ( 14 , :alphanum ) 83 iex&gt; Puid.Entropy . bits_for_len! ( 14 , &quot;dingosky&quot; ) 42","ref":"Puid.Entropy.html#bits_for_len!/2","title":"Puid.Entropy.bits_for_len!/2","type":"function"},{"doc":"See Puid.Entropy.bits_for_len/2 .","ref":"Puid.Entropy.html#bits_for_length/2","title":"Puid.Entropy.bits_for_length/2","type":"function"},{"doc":"See Puid.Entropy.bits_for_len!/2 .","ref":"Puid.Entropy.html#bits_for_length!/2","title":"Puid.Entropy.bits_for_length!/2","type":"function"},{"doc":"Entropy bits per character where charset is either an pre-defined Puid.CharSet or a string of unique characters. The character set must be comprised of unique symbols, and it is assumed each symbol in the character set has equal probability of occurrence (which maximizes entropy). Returns {:ok, bits} ; or {:error, reason} if arg is either an unrecognized pre-defined Puid.CharSet or a string of non-unique characters. Example iex&gt; Puid.Entropy . bits_per_char ( :alphanum ) { :ok , 5.954196310386875 } iex&gt; Puid.Entropy . bits_per_char ( &quot;dingosky&quot; ) { :ok , 3.0 }","ref":"Puid.Entropy.html#bits_per_char/1","title":"Puid.Entropy.bits_per_char/1","type":"function"},{"doc":"Same as bits_per_char/1 but either returns the bits or raises a Puid.Error Example iex&gt; Puid.Entropy . bits_per_char! ( :alphanum ) 5.954196310386875 Puid.Entropy . bits_per_char! ( &quot;dingosky&quot; ) 3.0","ref":"Puid.Entropy.html#bits_per_char!/1","title":"Puid.Entropy.bits_per_char!/1","type":"function"},{"doc":"Length needed for a string generated from charset to have bits of entropy. The character set must be comprised of unique symbols, and it is assumed each symbol in the character set has equal probability of occurrence (which maximizes entropy). Example iex&gt; Puid.Entropy . len_for_bits ( 128 , :alphanum ) { :ok , 22 } iex&gt; Puid.Entropy . len_for_bits ( 128 , &quot;dingosky&quot; ) { :ok , 43 }","ref":"Puid.Entropy.html#len_for_bits/2","title":"Puid.Entropy.len_for_bits/2","type":"function"},{"doc":"Same as Puid.Entropy.len_for_bits/2 but either returns the integer len or raises a Puid.Error Example iex&gt; Puid.Entropy . len_for_bits! ( 128 , :alphanum ) 22 iex&gt; Puid.Entropy . len_for_bits! ( 128 , &quot;dingosky&quot; ) 43","ref":"Puid.Entropy.html#len_for_bits!/2","title":"Puid.Entropy.len_for_bits!/2","type":"function"},{"doc":"Information regarding Puid module parameterization The Puid.Info struct has the following fields: Field Description chars source character set charset pre-defined Puid.Charset or :custom entropy_bits entropy bits for generated puid s entropy_bits_per_char entropy bits per character for generated puid s ere puid entropy string representation efficiency length puid string length rand_bytes entropy source function iex&gt; defmodule ( SafeId , do : use ( Puid , total : 1.0e04 , risk : 1.0e12 , chars : &quot;thequickbrownfxjmpsvlazydg&quot; ) ) iex&gt; SafeId . info ( ) % Puid.Info { chars : &quot;thequickbrownfxjmpsvlazydg&quot; , charset : :custom , entropy_bits : 65.81 , entropy_bits_per_char : 4.7 , ere : 0.59 , length : 14 , rand_bytes : &amp; :crypto . strong_rand_bytes / 1 }","ref":"Puid.Info.html","title":"Puid.Info","type":"module"},{"doc":"Errors raised when defining a Puid module with invalid options","ref":"Puid.Error.html","title":"Puid.Error","type":"exception"}]